@@ -7,6 +7,7 @@ from distributed.client import futures_of
 from distributed.metrics import time
 from distributed.utils_test import gen_cluster, inc, cluster
 from distributed.utils_test import loop  # noqa F401
+from distributed.protocol import Serialized
 from tornado import gen
 
 
@@ -18,6 +19,7 @@ def test_publish_simple(s, a, b):
     data = yield c.scatter(range(3))
     out = yield c.publish_dataset(data=data)
     assert 'data' in s.extensions['publish'].datasets
+    assert isinstance(s.extensions['publish'].datasets['data']['data'], Serialized)
 
     with pytest.raises(KeyError) as exc_info:
         out = yield c.publish_dataset(data=data)
@@ -26,15 +28,35 @@ def test_publish_simple(s, a, b):
     assert "data" in str(exc_info.value)
 
     result = yield c.scheduler.publish_list()
-    assert result == ['data']
+    assert result == ('data',)
 
     result = yield f.scheduler.publish_list()
-    assert result == ['data']
+    assert result == ('data',)
 
     yield c.close()
     yield f.close()
 
 
+@gen_cluster(client=False)
+def test_publish_non_string_key(s, a, b):
+    c = yield Client((s.ip, s.port), asynchronous=True)
+    f = yield Client((s.ip, s.port), asynchronous=True)
+
+    try:
+        for name in [('a', 'b'), 9.0, 8]:
+            data = yield c.scatter(range(3))
+            out = yield c.publish_dataset(data, name=name)
+            assert name in s.extensions['publish'].datasets
+            assert isinstance(s.extensions['publish'].datasets[name]['data'], Serialized)
+
+            datasets = yield c.scheduler.publish_list()
+            assert name in datasets
+
+    finally:
+        c.close()
+        f.close()
+
+
 @gen_cluster(client=False)
 def test_publish_roundtrip(s, a, b):
     c = yield Client((s.ip, s.port), asynchronous=True)
@@ -165,26 +187,29 @@ def test_publish_bag(s, a, b):
 def test_datasets_setitem(loop):
     with cluster() as (s, _):
         with Client(s['address'], loop=loop) as client:
-            key, value = 'key', 'value'
-            client.datasets[key] = value
-            assert client.get_dataset('key') == value
+            for key in ['key', ('key', 'key'), 1]:
+                value = 'value'
+                client.datasets[key] = value
+                assert client.get_dataset(key) == value
 
 
 def test_datasets_getitem(loop):
     with cluster() as (s, _):
         with Client(s['address'], loop=loop) as client:
-            key, value = 'key', 'value'
-            client.publish_dataset(key=value)
-            assert client.datasets[key] == value
+            for key in ['key', ('key', 'key'), 1]:
+                value = 'value'
+                client.publish_dataset(value, name=key)
+                assert client.datasets[key] == value
 
 
 def test_datasets_delitem(loop):
     with cluster() as (s, _):
         with Client(s['address'], loop=loop) as client:
-            key, value = 'key', 'value'
-            client.publish_dataset(key=value)
-            del client.datasets[key]
-            assert key not in client.list_datasets()
+            for key in ['key', ('key', 'key'), 1]:
+                value = 'value'
+                client.publish_dataset(value, name=key)
+                del client.datasets[key]
+                assert key not in client.list_datasets()
 
 
 def test_datasets_keys(loop):
@@ -210,3 +235,23 @@ def test_datasets_iter(loop):
             client.publish_dataset(**{str(key): key for key in keys})
             for n, key in enumerate(client.datasets):
                 assert key == str(n)
+
+
+@gen_cluster(client=True)
+def test_pickle_safe(c, s, a, b):
+    c2 = yield Client(s.address, asynchronous=True,
+                     serializers=['msgpack'])
+    try:
+        yield c2.publish_dataset(x=[1, 2, 3])
+        result = yield c2.get_dataset('x')
+        assert result == (1, 2, 3)
+
+        with pytest.raises(TypeError):
+            yield c2.publish_dataset(y=lambda x: x)
+
+        yield c.publish_dataset(z=lambda x: x)  # this can use pickle
+
+        with pytest.raises(TypeError):
+            yield c2.get_dataset('z')
+    finally:
+        yield c2.close()
