@@ -8,10 +8,10 @@ import functools
 import gc
 from glob import glob
 import itertools
-import inspect
 import logging
 import logging.config
 import os
+import psutil
 import re
 import shutil
 import signal
@@ -40,14 +40,17 @@ from tornado.gen import TimeoutError
 from tornado.ioloop import IOLoop
 
 from .client import default_client, _global_clients
-from .compatibility import PY3, iscoroutinefunction, Empty, WINDOWS
+from .compatibility import PY3, Empty, WINDOWS, PY2
+from .comm.utils import offload
 from .config import initialize_logging
 from .core import connect, rpc, CommClosedError
 from .metrics import time
+from .process import _cleanup_dangling
 from .proctitle import enable_proctitle_on_children
 from .security import Security
 from .utils import (ignoring, log_errors, mp_context, get_ip, get_ipv6,
-                    DequeHandler, reset_logger_locks, sync)
+                    DequeHandler, reset_logger_locks, sync,
+                    iscoroutinefunction, thread_state)
 from .worker import Worker, TOTAL_MEMORY, _global_workers
 
 try:
@@ -64,6 +67,9 @@ logging_levels = {name: logger.level for name, logger in
                   if isinstance(logger, logging.Logger)}
 
 
+offload(lambda: None).result()  # create thread during import
+
+
 @pytest.fixture(scope='session')
 def valid_python_script(tmpdir_factory):
     local_file = tmpdir_factory.mktemp('data').join('file.py')
@@ -126,6 +132,18 @@ def loop():
         else:
             is_stopped.wait()
     del _global_workers[:]
+
+    start = time()
+    while set(_global_clients):
+        sleep(0.1)
+        assert time() < start + 5
+
+    _cleanup_dangling()
+
+    if PY2:  # no forkserver, so no extra procs
+        for child in psutil.Process().children(recursive=True):
+            child.terminate()
+
     _global_clients.clear()
 
 
@@ -620,7 +638,10 @@ def cluster(nworkers=2, nanny=False, worker_kwargs={}, active_rpc_timeout=1,
             else:
                 client.close()
 
-    assert not ws
+    start = time()
+    while list(ws):
+        sleep(0.01)
+        assert time() < start + 1, 'Workers still around after one second'
 
 
 @gen.coroutine
@@ -719,12 +740,6 @@ def end_cluster(s, workers):
     s.stop()
 
 
-def iscoroutinefunction(f):
-    if sys.version_info >= (3, 5) and inspect.iscoroutinefunction(f):
-        return True
-    return False
-
-
 def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
                 scheduler='127.0.0.1', timeout=10, security=None,
                 Worker=Worker, client=False, scheduler_kwargs={},
@@ -842,12 +857,15 @@ def gen_cluster(ncores=[('127.0.0.1', 1), ('127.0.0.1', 2)],
                         break
                     else:
                         sleep(0.01)
-                    if time() > start + 2:
+                    if time() > start + 5:
                         from distributed import profile
                         tid = bad[0]
                         thread = threading._active[tid]
                         call_stacks = profile.call_stack(sys._current_frames()[tid])
                         assert False, (thread, call_stacks)
+            _cleanup_dangling()
+            with ignoring(AttributeError):
+                del thread_state.on_event_loop_thread
             return result
 
         return test_func
@@ -992,7 +1010,7 @@ def assert_can_connect(addr, timeout=None, connection_args=None):
     within the given *timeout*.
     """
     if timeout is None:
-        timeout = 0.2
+        timeout = 0.5
     comm = yield connect(addr, timeout=timeout,
                          connection_args=connection_args)
     comm.abort()
@@ -1005,7 +1023,7 @@ def assert_cannot_connect(addr, timeout=None, connection_args=None, exception_cl
     within the given *timeout*.
     """
     if timeout is None:
-        timeout = 0.2
+        timeout = 0.5
     with pytest.raises(exception_class):
         comm = yield connect(addr, timeout=timeout,
                              connection_args=connection_args)
